#ifndef GALLERYCONTENTPROVIDER_H
#define GALLERYCONTENTPROVIDER_H

#include "gallerycoreexport.h"
#include <QObject>
class QAbstractItemModel;

//! Class GalleryContentProvider is an abstract class meant to be inherited by a concrete provider.
//! Inheriting the GalleryContentProvider doesn't limit how the content is retrieved.
//! It can be fetched from the tracker, from the disk or over the network e.g. UPnP.
//! It is up to the developer to decide where the content should be fetched.
//! In order to make a custom content provider inherit this class and implement queryContent().
class GALLERYCORE_EXPORT GalleryContentProvider : public QObject
{
    Q_OBJECT
public:
    enum ColumnIndex {
        InvalidColumn = -1, // Invalid enum member for checkings.
        UrlColumn = 0, // QUrl
        FileNameColumn = 1, // QString
        CreatedColumn = 2, // QDateTime
        MimeTypeColumn = 3, // QString
        FavoriteColumn = 4, // bool
        DurationColumn = 5, // int
        UrnColumn = 6, // QString
        VirtualItemColumn = 7, // bool
        CeilingColumn = 8 // Ceiling enum member for checkings
    };

    //! Constructor. Doesn't perform the actual content query. See queryContent() for more details.
    //! \param parent optional parent.
    explicit GalleryContentProvider(QObject* parent = 0);

    //! Destructor.
    virtual ~GalleryContentProvider();

    //! \return QAbstractItemModel containing content data.
    //! See QAbstractItemModel for further details how to implement a model.
    virtual QAbstractItemModel* model() const = 0;

    //! Abstract method which must be implemented in the concrete provider.
    //! This method should fetch all the data and create the model based on the data.
    //! Use ContentProvider::setModel(QAbstractItemModel*) for setting the model.
    //! \param limit the upper limit for the amount content that the provider can provide
    //! If the limit is smaller than zero, all the available content should be included to the model.
    virtual void queryContent(int limit) = 0;

    //! This method can be used to know whether all requested content was
    //! loaded by the provider. If a provider fetches data asynchronously,
    //! this should return \c true while the operation is in progress. In
    //! general this method will always return \c false.
    virtual bool isLoadingContent() const = 0;
};

#endif // GALLERYCONTENTPROVIDER_H
