#ifndef GALLERYCROPPER_H
#define GALLERYCROPPER_H

#include "gallerycoreexport.h"
#include "galleryeditplugin.h"

class QPainter;
class QStyleOptionGraphicsItem;
class QGraphicsSceneMouseEvent;
class GalleryCropperPrivate;
class QRect;

//! \class GalleryCropper
//! \brief GalleryCropper class is used to handle Crop edit operations
//!
class GALLERYCORE_EXPORT GalleryCropper : public GalleryEditPlugin
{
    Q_OBJECT
    Q_INTERFACES(GalleryEditPlugin)
public:
    //! GalleryCropper constructor
    //! \param parent Optional parent
    GalleryCropper(QObject* parent = 0);

    //! GalleryCropper destructor.
    virtual ~GalleryCropper();

    //! \reimp
    QString name() const;
    //! \reimp
    bool containsUi() const;
    //! \reimp
    bool zoomingAllowed() const;
    //! \reimp
    bool receiveMouseEvent(QGraphicsSceneMouseEvent * event);
    //! \reimp
    void paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget = 0);

    //! Crop aspect ratio types
    enum AspectRatioType
    {
        AR_Free = 0, //!< No aspect ratio restriction
        AR_Original, //!< Original aspect ratio
        AR_Square, //!< Square aspect ratio
        AR_4_3, //!< 4:3 aspect ratio
        AR_3_4, //!< 3:4 aspect ratio
        AR_3_2, //!< 3:2 aspect ratio
        AR_2_3, //!< 2:3 aspect ratio
        AR_16_9, //!< 16:9 aspect ratio
        AR_9_16, //!< 9:16 aspect ratio
        AR_Portrait, //!< Aspect ratio matching device portrait mode screen size
        AR_Landscape //!< Aspect ratio matching device landscape mode screen size
    };

    //! Sets crop aspect ratio type. This method changes the current crop aspect ratio.
    //! The current selection rectangle is changed so that it has as big intersected
    //! area with the old selection as possible. If the \p ratio is the same as current
    //! ratio or AR_Free this method has no effect on current selection.
    //! Default value is AR_Original.
    //!
    //! \param ratio Crop aspect ratio type
    void setCropAspectRatioType(AspectRatioType ratio);

    //! Returns current aspect ratio type
    //! \return Aspect ratio type
    AspectRatioType cropAspectRatioType() const;

    //! Sets the minimum selection size that is allowed
    //! Default value is QSize(16,16)
    //! \param size Minimum selection size
    void setMinimumSelectionSize(const QSize& size);

    //! Returns the minimum selection size that is allowed
    //! \return Minimum selection size
    QSize minimumSelectionSize() const;

    //! Returns the current selection in image coordinates
    //! \returns The current selection  in image coordinates
    QRect selectionRect() const;

    //! Returns the current selection in widget coondinates
    //! \returns The current selection in widget coondinates
    QRect widgetSelectionRect() const;

    //! Returns crop aspect ratio formed from type
    //! \return Crop aspect ratio
    qreal cropAspectRatio();

    //! \return true when handle is visible, otherwise false
    bool isHandleVisible() const;

public Q_SLOTS:

    //! \reimp
    void performEditOperation();
    //! \reimp
    void activate();
    //! \reimp
    void deactivate(bool confirm);

Q_SIGNALS:
    //! Emitted when cropper action was confirmed
    //! \param selectionRect Selected rectangle
    void cropConfirmed(const QRect& selectionRect);

    //! Emitted when cropper action was canceled
    void cropCanceled();

private:
    // Private members
    Q_DISABLE_COPY( GalleryCropper );
    Q_DECLARE_PRIVATE( GalleryCropper );
    GalleryCropperPrivate * d_ptr;

    friend class Ut_GalleryCropper;
};

#endif
