#ifndef GALLERYEDITPLUGIN_H
#define GALLERYEDITPLUGIN_H

#include "gallerycoreexport.h"
#include <QtPlugin>
#include <mnamespace.h>

class GalleryEditPluginPrivate;
class GalleryEditUiProvider;
class QGraphicsWidget;
class MAction;
class QGraphicsItem;
class QGraphicsSceneMouseEvent;
class QStyleOptionGraphicsItem;
class QPainter;
class QSize;

//! \brief GalleryEditPlugin is a base class for all the Gallery edit plugins.
class GALLERYCORE_EXPORT GalleryEditPlugin : public QObject
{
    Q_OBJECT
public:
    //! GalleryEditPlugin destructor.
    virtual ~GalleryEditPlugin();

    //! Get the name of the plugin
    //! \return The name of the plugin
    virtual QString name() const  = 0;

    //! Get the index of the plugin when displaying in the edit tools list.
    //! The edit plugin with smallest index is shown first.
    //! The default implementation returns the same value as GalleryEditUiProvider::editPluginCount()
    //! or zero if the GalleryEditUiProvider has not been set.
    //! \return the index of the plugin
    virtual int menuIndex() const;

    //! Get the icon ID of the plugin when displaying in the list.
    //! The default implementation returns an empty string.
    //! \return the icon ID of the plugin
    virtual QString iconID() const;

    //! Tells whether this plugin needs to draw its own UI or not.
    //! The plugin should return true if it cannot perform its operation without the end user's input.
    //! For example a cropping plugin needs get the crop area from the end user
    //! before it can perform the crop operation and should return true.
    //! The default implementation returns false.
    //! \return true if plugin contains UI, otherwise false
    virtual bool containsUi() const;

    //! Tells whether the undo button should be visible while the plugin is performing its edit operation.
    //! The default implementation returns false.
    //! \return true if the undo button should be visible, otherwise false
    virtual bool undoVisible() const;

    //! Tells whether the redo button should be visible while the plugin is performing its edit operation.
    //! The default implementation returns false.
    //! \return true if the redo button should be visible, otherwise false
    virtual bool redoVisible() const;

    //! Tells whether this plugin allows zooming.
    //! The default implementation returns false.
    //! \return true if the plugin allows zooming, otherwise false
    virtual bool zoomingAllowed() const;

    //! Method for entering mouse events into plugin
    //! Possibly reimplemented in the inherited classes
    //! \param event Mouse event
    //! \return true if event was consumed, otherwise false
    virtual bool receiveMouseEvent(QGraphicsSceneMouseEvent* event);

    //! Method for plugin specific painting.
    //! The actual plugin must reimplement this function if it needs to do any painting.
    //! \param painter QPainter
    //! \param option QStyleOptionGraphicsItem
    //! \param widget QWidget
    virtual void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget = 0);

    //! Get function for tool bar widget fixed size based on orientation.
    //! \param orientation the current display orientation
    //! \return the size of widget
    virtual const QSize toolBarWidgetSize(const M::Orientation& orientation) const;

    //! Reserved for future use to maintain compatibility.
    //! \param operation the identifier for the operation
    //! \param data the input data for the operation
    //! \return an operation specific return value
    virtual QVariant extensionOperation(int operation, const QVariant& data);

    //! Set the edit UI provider
    //! \param editUiProvider the edit UI provider
    void setEditUiProvider(GalleryEditUiProvider* editUiProvider);

    //! Get the edit UI provider
    //! \return the edit UI provider
    GalleryEditUiProvider* editUiProvider() const;

    //! Get the optional toolbar widget for the plugin
    //! \param parent optional parent for the widget
    //! \return toolbar widget for this plugin or 0 if it doesn't exist
    QGraphicsWidget* toolBarWidget(QGraphicsItem* parent = 0);

    //! Tells whether the toolbar widget should be visible in certain orientation
    //! The default implementation returns true.
    //! \param orientation Orientation to check
    //! \return true if the widget should be visible, otherwise false
    virtual bool toolBarWidgetVisible(M::Orientation orientation) const;

    //! Get the optional toolbar action for the plugin
    //! \param parent optional parent for the action
    //! \return toolbar action for this plugin or 0 if it doesn't exist
    MAction* toolBarAction(QObject *parent = 0);

    //! Helper method for sorting the GalleryEditPlugins according to their index
    //! \param plugin1 First plugin
    //! \param plugin2 Second plugin
    //! \return true if index of first plugin is smaller than second, otherwise false
    static bool indexLessThan(GalleryEditPlugin* plugin1, GalleryEditPlugin* plugin2);

public Q_SLOTS:
    //! Performs edit operation for the plugin.
    //! The plugin must emit signal editOperationPerformed()
    //! when it has finished its edit operation.
    virtual void performEditOperation() = 0;

    //! Called when the plugin is activated.
    virtual void activate();

    //! Called when the plugin is deactivated.
    //! NOTE: If reimplemented, the reimplementation must call
    //! the base class implementation after its own operations.
    //! \param confirmed true if edit operation was confirmed by the user, otherwise false
    virtual void deactivate(bool confirmed);

Q_SIGNALS:
    //! Emitted when the plugin is selected
    void selected();

    //! Emitted by the plugin when the edit operation has been completed.
    void editOperationPerformed();

    //! Emitted when deactivated
    void deactivated();

protected:
    //! GalleryEditPlugin constructor
    //! \param parent optional parent
    GalleryEditPlugin(QObject* parent = 0);

    //! Creates the toolbar bar widget
    //! Possibly reimplemented in inherited classes
    //! \param parent Optional parent
    virtual QGraphicsWidget* createToolBarWidget(QGraphicsItem* parent = 0);

    //! Creates the toolbar bar action
    //! Possibly reimplemented in inherited classes
    //! \param parent Optional parent
    virtual MAction* createToolBarAction(QObject* parent = 0);

private:
    // Private members
    Q_DISABLE_COPY(GalleryEditPlugin);
    Q_DECLARE_PRIVATE(GalleryEditPlugin);
    GalleryEditPluginPrivate* d_ptr;

    friend class Ut_GalleryEditPlugin;
};

Q_DECLARE_INTERFACE(GalleryEditPlugin, "Gallery.EditPlugin/1.0")

#endif // GALLERYEDITPLUGIN_H

