#ifndef GALLERYEDITUIPROVIDER_H
#define GALLERYEDITUIPROVIDER_H

#include <QPoint>
#include <QRect>
#include <QString>
#include <QHash>
#include <QSize>
#include <QVariant>
#include <QuillImageFilter>

//! \class GalleryEditUiProvider
//! \brief GalleryEditUiProvider is interface for Gallery's edit plugins
//! to interact with the actual widget that provides the UI
class GalleryEditUiProvider
{
public:
    //! Converts a screen coordinate point to an image coordinate
    //! \param point point in the screen coordinates
    //! \return point in image the coordinates
    virtual const QPoint convertScreenCoordToImageCoord(const QPoint& point) const = 0;

    //! Gets the full image size
    //! \return the full image size
    virtual const QSize fullImageSize() const = 0;

    //! Gets the image area in screen coordinates
    //! \return the image area
    virtual const QRect imageArea() const = 0;

    //! Gets the current zoom factor
    //! \return the current zoom factor
    virtual qreal zoomFactor() const = 0;

    //! Gets the current viewport offset that is the top left corner of the current viewport.
    //! \return the current viewport offset
    virtual QPoint viewPortOffset() const = 0;

    //! Forces the screen to be repainted.
    virtual void forceScreenRepaint() = 0;

    //! Runs an edit filter on currently active image
    //! \param filterName the name of the filter
    //! \param filterOptionHash the hash of possible filter options
    //! \param replace true replaces old filters in this subsession, false keeps them
    virtual void runEditFilter(const QString& filterName, const QHash<QuillImageFilter::QuillFilterOption,QVariant>& filterOptionHash, bool replace = false) = 0;

    //! Gets the count of edit plugins.
    //! This function can be used for example by a plugin in the implementation of
    //! GalleryEditPlugin::menuIndex() to decide the plugin's menu index.
    virtual int editPluginCount() const = 0;

    //! Reserved for future use to maintain compatibility.
    //! \param operation the identifier for the operation
    //! \param data the input data for the operation
    //! \return an operation specific return value
    virtual QVariant extensionOperation(int operation, const QVariant& data) = 0;
};

#endif // GALLERYEDITUIPROVIDER_H
