#ifndef GALLERYFULLSCREENPAGE_H
#define GALLERYFULLSCREENPAGE_H

#include "gallerypage.h"

class GalleryFullScreenPagePrivate;
class MOrientationChangeEvent;

//! \class GalleryFullScreenPage
//! GalleryFullScreenPage is a GalleryPage which displays media items like images and videos
//! in fullscreen mode so that one item at a time is visible.
class GALLERYCORE_EXPORT GalleryFullScreenPage : public GalleryPage
{
    Q_OBJECT
public:
    //! Constructor.
    //! \param galleryModel the model that contains the data for the items shown in the page
    GalleryFullScreenPage(GalleryModel& galleryModel);
    //! Constructor.
    //! \param galleryModel the model that contains the data for the items shown in the page
    //! \param extensionActions the extension actions provided by the user of page
    //! The user of the page keeps the ownership of the actions.
    //! They should not be deleted before the page is also deleted.
    GalleryFullScreenPage(GalleryModel& galleryModel, const QList<MAction*>& extensionActions);

    //! Destructor.
    virtual ~GalleryFullScreenPage();

    //! Crop aspect ratio types
    enum CropAspectRatio
    {
        SquareAspectRatio = 0, //!< Square aspect ratio
        PortraitScreenAspectRatio //!< Aspect ratio matching device portrait mode screen size
    };

    //! Checks whether editing the items is enabled. By default it is disabled. \sa setEditingEnabled
    //! \return true if editing has been enabled, otherwise false.
    bool isEditingEnabled() const;

    //! Checks whether the page is currently in edit mode (the end user is editing the active item).
    //! \return true if the edit mode is active, otherwise false.
    bool isEditModeActive() const;

    //! Returns the paths where the edit plugins are searched from.
    //! \return edit plugin search paths.
    QStringList editPluginSearchPaths() const;

    //! Replaces the paths where the edit plugins will be searched from.
    //! Note that if this function has not been called, function editPluginSearchPaths()
    //! will return the default paths and this functions replaces them.
    //! This means that if new plugin serach paths need to be added to the default ones,
    //! the new paths should be appended to the result of editPluginSearchPaths().
    //! \param paths the plugin search paths that will replace the existing ones.
    void setEditPluginSearchPaths(const QStringList& paths);

    //! Returns the URL of the currently active item on the page.
    //! \return the URL of the active item
    QUrl activeItemUrl() const;

    //! \reimp
    virtual bool event(QEvent *event);

    //! Return true: if the item is read-only and not-indexed by tracker
    bool canActiveItemBeShared() const;

    //! \return Crop aspect ratio
    CropAspectRatio cropAspectRatio() const;

    //! Set function for aspect ratio used in cropping.
    //! By default this value is SquareAspectRatio.
    //! \param aspectRatio Crop aspect ratio
    void setCropAspectRatio(CropAspectRatio aspectRatio);

public Q_SLOTS:
    //! \reimp
    virtual void moveToItem(const QUrl& itemUrl);
    //! \reimp
    virtual void resetToDefaultState();
    //! \reimp
    void setNavigationBarVisible(bool value);

    //! If the active item is video, starts playing the video from the beginning.
    //! If active item is image, or video is already playing, calling this function does nothing.
    void playActiveItem();

    //! If the active item is video and it is currently playing, stops video playback and resets it to beginning.
    //! Otherwise this function does nothing.
    void stopVideoPlayback();

    //! Sets editing the items enabled.
    //! By default editing is disabled, so the user needs to call this function to enable editing.
    //! \param enabled true enables editing and false disables it
    void setEditingEnabled(bool enabled);

    //! Opens the currently active item for editing.
    void openActiveItemInEditMode();

    //! Sets the slide show speed.
    //! Slideshow speed is the interval used for switching from the current slide to the next slide.
    //! \param slideShowSpeed slideshow speed in milliseconds.
    void setSlideshowSpeed(int slideshowSpeed);

    //! \reimp
    void orientationChangeEvent(MOrientationChangeEvent* event);

    //! Moves to the item given and opens it in cropper.
    //! When the cropping has been done signal croppingDone() gets emitted.
    //! Does not modify the original item, but just requests the user to crop an area from it.
    //! Uses square aspect ratio by default unless otherwise specified with setCropAspectRatio()
    //! \param itemUrl the URL of the item to be opened in cropper
    void openItemInCropper(const QUrl& itemUrl);

    //! Unloads the active item and the other possible cached items.
    //! This function should be called to stop all the background activity
    //! and release memory for the page when the page is not used / visible.
    void clearActiveItem();

    //! Saves the active item with a different name.
    //! The user will be prompted for the name and signal itemSaved will be emitted.
    void saveActiveItem();

    //! Opens the item with cropper and sets it as a wallpaper
    //! \param url the URL of the item
    void setAsWallpaper(const QUrl& url);

Q_SIGNALS:
    //! The signal is emitted whenever the user interacts with the page.
    void userActivity();

    //! The signal is emitted when play button is pressed
    void aboutToPlayVideo();

    //! The signal is emitted when the user has finished cropping an item in the cropper view.
    //! \param croppedImage the image cropped from the original item
    void croppingDone(QImage croppedImage);

    //! The signal is emitted when the user has cancelled cropping an item in the cropper view.
    void croppingCancelled();

    //! Emitted when loading of active item has failed
    //! \param errorNotification translated error string
    //! \note Showing the message properly in UI requires loading gallery catalog
    void loadingActiveItemFailed(const QString& errorNotification);

    //! Emitted when an item has been saved as a result of calling saveActiveItem() with a new name.
    //! \param originalUrl the original URL of the item
    //! \param newUrl the new URL of the item
    void itemSaved(const QUrl& originalUrl, const QUrl& newUrl);

protected:
    //! For internal use.
    void addToPreviewImageList(const QUrl& url, const QImage image, const QMultiMap<int,QVariant>& imageParams);
    //! For internal use.
    QList<QUrl> previewImageList() const;
    //! For internal use.
    void previewImageSaved(const QUrl& url);
    //! For internal use.
    void removeFromPreviewImageList(const QUrl& url);

private:
    Q_DISABLE_COPY(GalleryFullScreenPage);
    Q_DECLARE_PRIVATE(GalleryFullScreenPage);

    friend class Ut_GalleryFullScreenPage;
};

#endif // GALLERYFULLSCREENPAGE_H
