#ifndef GALLERYGRIDPAGE_H
#define GALLERYGRIDPAGE_H

#include "gallerypage.h"

class GalleryGridPagePrivate;

//! GalleryGridPage is a GalleryPage which displays media items like images and videos
//! as thumbnails in grid mode.
class GALLERYCORE_EXPORT GalleryGridPage : public GalleryPage
{
    Q_OBJECT
public:
    //! Constructor.
    //! \param galleryModel the model that contains the data for the items shown in the page
    GalleryGridPage(GalleryModel& galleryModel);
    //! Constructor.
    //! \param galleryModel the model that contains the data for the items shown in the page
    //! \param extensionActions the extension actions provided by the user of page
    //! The user of the page keeps the ownership of the actions.
    //! They should not be deleted before the page is also deleted.
    GalleryGridPage(GalleryModel& galleryModel, const QList<MAction*>& extensionActions);
    //! Destructor.
    virtual ~GalleryGridPage();

    //! Set a text to the top bar in a grid. Note this doesn't show the top bar yet,
    //! you need to call showTopBar(true) to make it visible.
    //! \param text A text for a Grid's top bar.
    void setTopBarText(QString const & text);

    //! Make Grid's topBar visible.
    //! \param show True makes top bar visible, false hides it.
    void showTopBar(bool show);

    //! Moves the page into single selection mode.
    //! In single selection mode exactly one item can be selected and
    //! all other actions on the page are disabled.
    void selectItem();

    //! The list of selected items in multiselection mode.
    //! In the default mode the returned list is empty.
    //! In multiselection mode the list contains the items the user has currently selected.
    //! \return the list of selected item URLs
    QList<QUrl> selectedItems() const;

    //! \return true if the page is in sharing state.
    bool isInSharingState() const;

public Q_SLOTS:
    //! \reimp
    virtual void resetToDefaultState();

    //! Moves the page to multiselection mode.
    //! \param confirmationString the text to be shown on the confirmation button for the multiselection
    void startMultiSelection(const QString& confirmationString);

    //! Move the focus to the item identified by the URL
    //! \param itemUrl the URL of the item
    //! \param highlight Whether focused item should be highlighted or not
    void moveToItem(const QUrl& itemUrl, bool highlight);

Q_SIGNALS:
    //! The signal is emitted when an item should be opened for editing
    //! \param url the URL of the item
    void editItem(const QUrl& url);

    //! The signal is emitted when the end user has indicated the multiselection is done.
    void multiSelectionDone(QList<QUrl> selectedItems);

    //! The signal is emitted if the end user has cancelled the multiselection.
    void multiSelectionCancelled();

    //! The signal is emitted if the end user has cancelled the single selection.
    void singleSelectionCancelled();

    //! The signal is emitted when an image should be set as wallpaper
    //! \param url the URL of the image
    void setAsWallpaper(const QUrl& url);

private:
    Q_DISABLE_COPY(GalleryGridPage);
    Q_DECLARE_PRIVATE(GalleryGridPage);

    friend class Ut_GalleryGridPage;
};

#endif // GALLERYGRIDPAGE_H
