#ifndef GALLERYITEMDATA_H
#define GALLERYITEMDATA_H

#include "gallerycoreexport.h"
#include <QUrl>
#include <QDateTime>

class GalleryItemDataPrivate;

//! \class GalleryItemData
//! \brief GalleryItemData is a container for the data of one gallery
//!        item (image or video)
class GALLERYCORE_EXPORT GalleryItemData
{

public:

    //! Permissions for actions on the item
    enum Permission {
        None = 0x0, //!< None pemission granted on item
        Delete = 0x1, //!< Deleting is granted
        Modify = 0x2, //!< Modifying is granted
        Save = 0x4, //!< Saving is granted
        Share = 0x8, //!< Sharing is granted
        Tag = 0x10 //!< Tagging is granted
    };
    Q_DECLARE_FLAGS(Permissions, Permission)

    //! Constructs an empty MediaItem.
    GalleryItemData();

    //! Copy constructor.
    //! \param other the item to be copied
    GalleryItemData(const GalleryItemData& other);

    //! Destructor.
    virtual ~GalleryItemData();

    //! Assigment operator
    //! \param other the item from which the data will be copied to this item
    //! \return reference to the updated item
    GalleryItemData& operator=(const GalleryItemData& other);

    //! Checks whether the item is empty (does not contain any data)
    //! \return true if the item is empty, otherwise false
    bool isEmpty() const;

    //! Gets the URL of the item.
    //! \return The URL of the item.
    QUrl url() const;

    //! Gets the tracker urn of the item.
    //! \return Tracker urn of the item.
    QString urn() const;

    //! Gets the creation time and date of media item (image or video)
    //! \return the creation time and date
    QDateTime created() const;

    //! Gets the item mimetype
    //! \return the mimetype of the item
    QString mimeType() const;

    //! Checks whether media item is favorite
    //! \return true if item is favorite
    bool favorite() const;

    //! Gets the duration of the (video) item
    //! \return Duration of the (video) item or 0 if image etc.
    int duration() const;

    //! Checks whether loading the media item has failed.
    //! \return true if loading the item has failed
    bool loadingFailed() const;

    //! Checks whether the passed permissions are granted to the item
    //! or not.
    //!
    //! This function is more convenient than asking for the
    //! permissions since, depending on the actual required check, it
    //! can be faster.
    //!
    //! \param permission to check.
    //! \return true if the permissions are granted, false otherwise.
    bool hasPermission(const GalleryItemData::Permissions permission) const;

    //! Provides the permissions granted to the item.
    //!
    //! \return the granted permissions to the item.
    GalleryItemData::Permissions itemPermissions() const;

private:
    mutable QSharedDataPointer<GalleryItemDataPrivate> d_ptr;
    friend class GalleryModel;
    friend class FullScreenMediaWidget;
    friend class FullScreenQuillFileHandlerPrivate;
    friend class GridViewPrivate;
    friend class Ut_FullScreenQuillFileHandler;
    friend class Ut_GridView;
    friend class Ut_GalleryItemData;
};

Q_DECLARE_OPERATORS_FOR_FLAGS(GalleryItemData::Permissions)

#endif // GALLERYITEMDATA_H
