#ifndef GALLERYMODEL_H
#define GALLERYMODEL_H

#include "gallerycoreexport.h"
#include "galleryitemdata.h"
#include <QObject>
#include <QList>
#include <QUrl>
#include <QModelIndex>

class GalleryContentProvider;
class GalleryModelPrivate;
class QSparqlConnection;


//! \class GalleryModel
//! \brief GalleryModel provides the data model for the Gallery UI.
class GALLERYCORE_EXPORT GalleryModel : public QObject
{
    Q_OBJECT
public:

    //! Constructor.
    //! \param parent optional parent
    GalleryModel(QObject* parent = 0);

    //! Destructor
    virtual ~GalleryModel();

    //! Add a new GalleryContentProvider object to this model.
    //! \param provider a GalleryContentProvider object, the ownership is not transferred.
    //! \param queryLimit the upper limit for the amount content that the provider should provide
    //! If the value is smaller than zero, all the available content should be included to the resulting model.
    void addContentProvider(GalleryContentProvider* provider, int queryLimit=-1);

    //! Removes a GalleryContentProvider object from this model.
    //! \param provider a GalleryContentProvider object
    void removeContentProvider(GalleryContentProvider* provider);

    //! Get the content provider in the model
    //! \return the content provider
    GalleryContentProvider* contentProvider() const;

    //! \return the size of the model.
    int itemCount() const;

    //! \return count of failed urls
    int failedUrlsCount() const;

    //! Gets a media item by its index.
    //! The caller should not cache the item data
    //! as the returned instance is not updated when the model is updated.
    //! \param index the index of the item
    //! \return the media item found or an empty media item if not found (see GalleryItemData::isEmpty())
    GalleryItemData itemAt(int index) const;

    //! Gets an item by its URL.
    //! The caller should not cache the item data
    //! as the returned instance is not updated when the model is updated.
    //! \param url the URL of the item
    //! \return the item found or an empty item if not found (see GalleryItemData::isEmpty())
    GalleryItemData findItem(const QUrl& url) const;

    //! Gets the index of an item by its URL.
    //! \param url the URL of the item
    //! \return the index of the item with the URL or -1 if not found
    int indexOfItem(const QUrl& url) const;

    //! Finds next item starting from given url
    //! \param url Url from to start searching
    //! \param wrapAround If set to true, searching will continue from the beginning if reaches the end of list
    //! \param skipFailed If set to true, will skip items whose loading has failed
    //! \return Found mediaitem or empty item if not found (see GalleryItemData::isEmpty())
    GalleryItemData nextItem(const QUrl& url, bool wrapAround=true, bool skipFailed=true) const;

    //! Finds previous item starting from given url
    //! \param url Url from to start searching
    //! \param wrapAround If set to true, searching will continue from the end if reaches the beginning of list
    //! \param skipFailed If set to true, will skip items whose loading has failed
    //! \return Found mediaitem or empty item if not found (see GalleryItemData::isEmpty())
    GalleryItemData previousItem(const QUrl& url, bool wrapAround=true,bool skipFailed=true) const;

    //! Removes a media item by its URL from the model.
    //! \param url the URL of the item
    //! \return true if the item was removed, otherwise false
    bool removeItem(const QUrl& url);

    //! Removes a list of media item by its URL from the model.
    //! \param itemUrls the list of URL of the items
    //! \return true if the items were removed, otherwise false
    bool removeItems(const QList<QUrl>& itemUrls);

    //! Sets the flag that loading the media item has failed.
    //! \param itemUrl the URL of the item
    //! \param failed true, if loading the item has failed.
    void setLoadingFailed(const QUrl& itemUrl, bool failed);

    //! Gets an item's favorite status.
    //! Note that GalleryItemData::favorite() may not return the up-to-date information
    //! as it is not dynamically updated after the item has been added to the model.
    //! Use this function to get the up-to-date information about the favorite status.
    //! \return true if the item was found from the model and marked as favorite, otherwise false
    bool isMarkedAsFavorite(const QUrl& itemUrl);

    //! Check whether some GalleryContentProvider is still loading items.
    //! \return true if there is at least one GalleryContentProvider for
    //! which isLoadingContent() returns true, false otherwise (see
    //! GalleryContentProvider::isLoadingContent() for details)
    bool isLoadingContent() const;

    //! Obtain the connection used by the model to access the Tracker store.
    //! If a connection is not set using \ref setSparqlConnection(),
    //! a connection with default driver and options will be created.
    QSparqlConnection* sparqlConnection();

    //! Change the connection used by the model to access the Tracker store.
    //! This allows setting a connection different than the default one,
    //! possibly with different connection options. The caller will still
    //! be responsible for closing the connection after the model is freed.
    void setSparqlConnection(QSparqlConnection *connection);

    //! Set thumbnail creation enabled. Enabled by default.
    //! The thumbnail creation should be disabled if the items in the content provider
    //! are temporary in that sense that they are not and will not be indexed by Tracker.
    //! \param enabled if true enables thumbnail creation. If false no thumbnails are created.
    void setThumbnailCreationEnabled(bool enabled);

    //! Set external video thumbnailer enabled. Enabled by default.
    //! \note If set to disabled, videos will not get thumbnails generated via gallerycore.
    //! \param enabled if true enables support for external video thumbnailing service
    void setExternalVideoThumbnailerEnabled(bool enabled);

    //! Returns the device model using SystemDeviceInfo
    //! and taking into account whether the device is still a prototype
    //! or not
    //! \return the device model
    static QString deviceModel();

public Q_SLOTS:
    //! Sets the item marked as a favorite
    //! \param itemUrl the URL of the item
    //! \param favorite true to set the item as a favorite, false to remove the favorite marking
    void setAsFavorite(const QUrl& itemUrl, bool favorite=true);

Q_SIGNALS:
    //! A signal for indicating that items have been added to the model
    //! \param first the first index where a new item was added
    //! \param last the last index where a new item was added
    void galleryItemsAdded(int first, int last);

    //! A signal for indicating that items have been removed from the model
    //! \param first the first index that was removed from the model
    //! \param last the last index that was removed from the model
    void galleryItemsRemoved(int first, int last);

    //! A Signal for indicating that the favorite status has changed for an item.
    //! \param itemUrl the URL of the item
    //! \param favorite the current favorite status
    void galleryItemFavoriteStatusChanged(const QUrl& itemUrl, bool favorite);

    //! A Signal for indicating that the virtual status has changed for an item.
    //! \param itemUrl the URL of the item
    //! \param the current virtual status
    //! \param itemUrn the URN of the item
    void galleryItemVirtualStatusChanged(const QUrl& itemUrl, bool isVirtual, const QString& itemUrn);

    //! A signal for indicating that an item has changed in such a way that
    //! all cached data about it should be refreshed.
    //! \param itemUrl the URL of the item.
    void galleryItemChanged(const QUrl& itemUrl);

    //! A Signal for indicating that all the content providers in the model have
    //! been removed.
    void allContentProvidersRemoved();

public Q_SLOTS:
    void rowsInserted(const QModelIndex& parent, int start, int end);
    void rowsRemoved(const QModelIndex& parent, int start, int end);

private:
    Q_DISABLE_COPY(GalleryModel);
    Q_DECLARE_PRIVATE(GalleryModel);
    GalleryModelPrivate* d_ptr;

    friend class Ut_GalleryModel;
    friend class GalleryCoreSimulator;
};

#endif // GALLERYMODEL_H
