#ifndef GALLERYPAGE_H
#define GALLERYPAGE_H

#include "gallerycoreexport.h"
#include <MApplicationPage>
#include <QString>
#include <QUrl>

class GalleryModel;
class MSheet;
class GalleryPagePrivate;
class ShareUiInterface;

//! \class GalleryPage
//! GalleryPage is a base class for displaying media items like images and videos.
class GALLERYCORE_EXPORT GalleryPage : public MApplicationPage
{
    Q_OBJECT
public:
    //! Destructor.
    virtual ~GalleryPage();

    //! \reimp
    virtual void createContent();

    //! Gets the model used in the page.
    //! \return the model
    GalleryModel& galleryModel();

    //! Returns the sheet that can be used for showing the same central widget as the page.
    //! If the user wants to represent the page's content in a MSheet instead of MApplicationPage,
    //! it should call GalleryPage::sheet().appear() rather than GalleryPage::appear().
    MSheet& sheet();

    //! Sets the visibility of the navigation bar
    //! \param value Visibility value
    virtual void setNavigationBarVisible(bool value);

    //! Gets the visibility of the navigation bar
    //! \return Visibility value
    bool isNavigationBarVisible() const;

    //! Gets the count of items (images and videos) in this page
    //! \return the count of items
    int itemCount() const;

    //! Sets the sharing provider used when the user wants to share a media item.
    //! This function must be called if the user of the page wants the sharing option to be available.
    //! \param shareUiInterface the sharing provider used. Passing null pointer removes the previously set value.
    void setSharingProvider(ShareUiInterface* shareUiInterface);

    //! Returns a list of extension actions
    //! \return List of extension actions
    QList<MAction*> extensionActions() const;

    //! Sets a list of extension actions
    //! \param actions List of extension actions
    void setExtensionActions(const QList<MAction*>& actions);

    //! Instruct the page to run on low or normal memory consumption mode.
    //! If the memory saving mode is explicitly set to true, the page will
    //! keep the memory consumption low until the mode is set back to false.
    //! If the memory saving mode is explicitly set to false, the page will
    //! lower the memory consumption independently depending on its own
    //! display presence or the dislay presence of the whole application.
    //! \param active true for low memory mode, false for normal mode.
    void setMemorySavingMode(bool active);

    //! Check if memory saving mode has been explicitly set.
    bool isMemorySavingModeActive() const;

    //! Checks whether the page is in its default state
    //! (for example editing or selection state is not active).
    //! \return true if the page is in the default state.
    bool isInDefaultState() const;

public Q_SLOTS:
    //! Move the focus to the item identified by the URL
    //! \param itemUrl the URL of the item
    virtual void moveToItem(const QUrl& itemUrl);

    //! Called to reset page back to default state
    virtual void resetToDefaultState();

Q_SIGNALS:
    //! The signal is emitted when the active media item has been changed
    //! param itemUrl the URL of the media item
    void itemSelected(const QUrl& itemUrl);

    //! The signal is emitted when a media item in the page has been deleted
    //! param itemUrl the URL of the media item
    void itemDeleted(const QUrl& itemUrl);

protected:
    //! Constructor.
    GalleryPage(GalleryPagePrivate* dd, const QString& pageId);
    GalleryPagePrivate* const d_ptr;

private:
    Q_DISABLE_COPY(GalleryPage);
    Q_DECLARE_PRIVATE(GalleryPage);

    friend class GalleryFullScreenPagePrivate;
    friend class GalleryGridPagePrivate;
    friend class Ut_GalleryPage;
};

#endif // GALLERYPAGE_H
